"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import google.protobuf.timestamp_pb2
import job_pb2
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class Serving(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    class _DeploymentType:
        ValueType = typing.NewType('ValueType', builtins.int)
        V: typing_extensions.TypeAlias = ValueType
    class _DeploymentTypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[Serving._DeploymentType.ValueType], builtins.type):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        UNKNOWN: Serving._DeploymentType.ValueType  # 0
        """Unknown deployment type"""

        NBOX_SERVING: Serving._DeploymentType.ValueType  # 1
        """vanilla nbox serving with fastapi"""

    class DeploymentType(_DeploymentType, metaclass=_DeploymentTypeEnumTypeWrapper):
        """Deployment type"""
        pass

    UNKNOWN: Serving.DeploymentType.ValueType  # 0
    """Unknown deployment type"""

    NBOX_SERVING: Serving.DeploymentType.ValueType  # 1
    """vanilla nbox serving with fastapi"""


    ID_FIELD_NUMBER: builtins.int
    NAME_FIELD_NUMBER: builtins.int
    AUTH_INFO_FIELD_NUMBER: builtins.int
    CREATED_AT_FIELD_NUMBER: builtins.int
    TYPE_FIELD_NUMBER: builtins.int
    RESOURCE_FIELD_NUMBER: builtins.int
    id: typing.Text
    """ID for the deployment"""

    name: typing.Text
    """name of the deployment"""

    @property
    def auth_info(self) -> job_pb2.NBXAuthInfo:
        """string workspace_id = 3; // Workspace ID, if None assume in user workspace
        auth info for the deployment
        """
        pass
    @property
    def created_at(self) -> google.protobuf.timestamp_pb2.Timestamp:
        """time created"""
        pass
    type: global___Serving.DeploymentType.ValueType
    @property
    def resource(self) -> job_pb2.Resource:
        """deployment resource"""
        pass
    def __init__(self,
        *,
        id: typing.Text = ...,
        name: typing.Text = ...,
        auth_info: typing.Optional[job_pb2.NBXAuthInfo] = ...,
        created_at: typing.Optional[google.protobuf.timestamp_pb2.Timestamp] = ...,
        type: global___Serving.DeploymentType.ValueType = ...,
        resource: typing.Optional[job_pb2.Resource] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["auth_info",b"auth_info","created_at",b"created_at","resource",b"resource"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["auth_info",b"auth_info","created_at",b"created_at","id",b"id","name",b"name","resource",b"resource","type",b"type"]) -> None: ...
global___Serving = Serving
