# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: nbox_ws.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import job_pb2 as job__pb2
from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2
from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2
from google.protobuf import field_mask_pb2 as google_dot_protobuf_dot_field__mask__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\rnbox_ws.proto\x12\x07jobs.ws\x1a\tjob.proto\x1a\x1bgoogle/protobuf/empty.proto\x1a\x1egoogle/protobuf/wrappers.proto\x1a google/protobuf/field_mask.proto\x1a\x1fgoogle/protobuf/timestamp.proto\"1\n\x07JobInfo\x12\x0e\n\x02id\x18\x01 \x01(\tB\x02\x18\x01\x12\x16\n\x03job\x18\x02 \x01(\x0b\x32\t.jobs.Job\"^\n\x0fListJobsRequest\x12$\n\tauth_info\x18\x01 \x01(\x0b\x32\x11.jobs.NBXAuthInfo\x12\x11\n\tpage_size\x18\x02 \x01(\x05\x12\x12\n\npage_token\x18\x03 \x01(\t\"D\n\x10ListJobsResponse\x12\x17\n\x04Jobs\x18\x01 \x03(\x0b\x32\t.jobs.Job\x12\x17\n\x0fnext_page_token\x18\x02 \x01(\t\"L\n\x11UploadCodeRequest\x12\x16\n\x03job\x18\x01 \x01(\x0b\x32\t.jobs.Job\x12\x1f\n\x04\x61uth\x18\x02 \x01(\x0b\x32\x11.jobs.NBXAuthInfo\"*\n\x10\x43reateJobRequest\x12\x16\n\x03job\x18\x03 \x01(\x0b\x32\t.jobs.Job\"[\n\x10UpdateJobRequest\x12\x16\n\x03job\x18\x01 \x01(\x0b\x32\t.jobs.Job\x12/\n\x0bupdate_mask\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.FieldMask\"_\n\x0eJobLogsRequest\x12\x1d\n\x03job\x18\x01 \x01(\x0b\x32\x10.jobs.ws.JobInfo\x12\x1f\n\x08k8s_info\x18\x02 \x01(\x0b\x32\r.jobs.K8sInfo\x12\r\n\x05lines\x18\x03 \x01(\x05\"U\n\x06JobLog\x12\x1d\n\x03job\x18\x01 \x01(\x0b\x32\x10.jobs.ws.JobInfo\x12\x1f\n\x08k8s_info\x18\x02 \x01(\x0b\x32\r.jobs.K8sInfo\x12\x0b\n\x03log\x18\x03 \x03(\t\"z\n\x10UpdateRunRequest\x12\r\n\x05token\x18\x01 \x01(\t\x12\x16\n\x03job\x18\x02 \x01(\x0b\x32\t.jobs.Job\x12.\n\nupdated_at\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x0f\n\x07message\x18\x04 \x01(\t2\xd4\x04\n\x0cWSJobService\x12\'\n\x06GetJob\x12\x10.jobs.ws.JobInfo\x1a\t.jobs.Job\"\x00\x12\x41\n\x08ListJobs\x12\x18.jobs.ws.ListJobsRequest\x1a\x19.jobs.ws.ListJobsResponse\"\x00\x12\x38\n\rUploadJobCode\x12\x1a.jobs.ws.UploadCodeRequest\x1a\t.jobs.Job\"\x00\x12\x33\n\tCreateJob\x12\x19.jobs.ws.CreateJobRequest\x1a\t.jobs.Job\"\x00\x12\x33\n\tUpdateJob\x12\x19.jobs.ws.UpdateJobRequest\x1a\t.jobs.Job\"\x00\x12\x37\n\tDeleteJob\x12\x10.jobs.ws.JobInfo\x1a\x16.google.protobuf.Empty\"\x00\x12:\n\nGetJobLogs\x12\x17.jobs.ws.JobLogsRequest\x1a\x0f.jobs.ws.JobLog\"\x00\x30\x01\x12<\n\nTriggerJob\x12\x10.jobs.ws.JobInfo\x1a\x1a.google.protobuf.BoolValue\"\x00\x12@\n\tUpdateRun\x12\x19.jobs.ws.UpdateRunRequest\x1a\x16.google.protobuf.Empty\"\x00\x12?\n\x0bGetRunToken\x12\x10.jobs.ws.JobInfo\x1a\x1c.google.protobuf.StringValue\"\x00\x42\x32Z0github.com/NimbleBoxAI/jobs-architecture/jobs_pbb\x06proto3')



_JOBINFO = DESCRIPTOR.message_types_by_name['JobInfo']
_LISTJOBSREQUEST = DESCRIPTOR.message_types_by_name['ListJobsRequest']
_LISTJOBSRESPONSE = DESCRIPTOR.message_types_by_name['ListJobsResponse']
_UPLOADCODEREQUEST = DESCRIPTOR.message_types_by_name['UploadCodeRequest']
_CREATEJOBREQUEST = DESCRIPTOR.message_types_by_name['CreateJobRequest']
_UPDATEJOBREQUEST = DESCRIPTOR.message_types_by_name['UpdateJobRequest']
_JOBLOGSREQUEST = DESCRIPTOR.message_types_by_name['JobLogsRequest']
_JOBLOG = DESCRIPTOR.message_types_by_name['JobLog']
_UPDATERUNREQUEST = DESCRIPTOR.message_types_by_name['UpdateRunRequest']
JobInfo = _reflection.GeneratedProtocolMessageType('JobInfo', (_message.Message,), {
  'DESCRIPTOR' : _JOBINFO,
  '__module__' : 'nbox_ws_pb2'
  # @@protoc_insertion_point(class_scope:jobs.ws.JobInfo)
  })
_sym_db.RegisterMessage(JobInfo)

ListJobsRequest = _reflection.GeneratedProtocolMessageType('ListJobsRequest', (_message.Message,), {
  'DESCRIPTOR' : _LISTJOBSREQUEST,
  '__module__' : 'nbox_ws_pb2'
  # @@protoc_insertion_point(class_scope:jobs.ws.ListJobsRequest)
  })
_sym_db.RegisterMessage(ListJobsRequest)

ListJobsResponse = _reflection.GeneratedProtocolMessageType('ListJobsResponse', (_message.Message,), {
  'DESCRIPTOR' : _LISTJOBSRESPONSE,
  '__module__' : 'nbox_ws_pb2'
  # @@protoc_insertion_point(class_scope:jobs.ws.ListJobsResponse)
  })
_sym_db.RegisterMessage(ListJobsResponse)

UploadCodeRequest = _reflection.GeneratedProtocolMessageType('UploadCodeRequest', (_message.Message,), {
  'DESCRIPTOR' : _UPLOADCODEREQUEST,
  '__module__' : 'nbox_ws_pb2'
  # @@protoc_insertion_point(class_scope:jobs.ws.UploadCodeRequest)
  })
_sym_db.RegisterMessage(UploadCodeRequest)

CreateJobRequest = _reflection.GeneratedProtocolMessageType('CreateJobRequest', (_message.Message,), {
  'DESCRIPTOR' : _CREATEJOBREQUEST,
  '__module__' : 'nbox_ws_pb2'
  # @@protoc_insertion_point(class_scope:jobs.ws.CreateJobRequest)
  })
_sym_db.RegisterMessage(CreateJobRequest)

UpdateJobRequest = _reflection.GeneratedProtocolMessageType('UpdateJobRequest', (_message.Message,), {
  'DESCRIPTOR' : _UPDATEJOBREQUEST,
  '__module__' : 'nbox_ws_pb2'
  # @@protoc_insertion_point(class_scope:jobs.ws.UpdateJobRequest)
  })
_sym_db.RegisterMessage(UpdateJobRequest)

JobLogsRequest = _reflection.GeneratedProtocolMessageType('JobLogsRequest', (_message.Message,), {
  'DESCRIPTOR' : _JOBLOGSREQUEST,
  '__module__' : 'nbox_ws_pb2'
  # @@protoc_insertion_point(class_scope:jobs.ws.JobLogsRequest)
  })
_sym_db.RegisterMessage(JobLogsRequest)

JobLog = _reflection.GeneratedProtocolMessageType('JobLog', (_message.Message,), {
  'DESCRIPTOR' : _JOBLOG,
  '__module__' : 'nbox_ws_pb2'
  # @@protoc_insertion_point(class_scope:jobs.ws.JobLog)
  })
_sym_db.RegisterMessage(JobLog)

UpdateRunRequest = _reflection.GeneratedProtocolMessageType('UpdateRunRequest', (_message.Message,), {
  'DESCRIPTOR' : _UPDATERUNREQUEST,
  '__module__' : 'nbox_ws_pb2'
  # @@protoc_insertion_point(class_scope:jobs.ws.UpdateRunRequest)
  })
_sym_db.RegisterMessage(UpdateRunRequest)

_WSJOBSERVICE = DESCRIPTOR.services_by_name['WSJobService']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z0github.com/NimbleBoxAI/jobs-architecture/jobs_pb'
  _JOBINFO.fields_by_name['id']._options = None
  _JOBINFO.fields_by_name['id']._serialized_options = b'\030\001'
  _JOBINFO._serialized_start=165
  _JOBINFO._serialized_end=214
  _LISTJOBSREQUEST._serialized_start=216
  _LISTJOBSREQUEST._serialized_end=310
  _LISTJOBSRESPONSE._serialized_start=312
  _LISTJOBSRESPONSE._serialized_end=380
  _UPLOADCODEREQUEST._serialized_start=382
  _UPLOADCODEREQUEST._serialized_end=458
  _CREATEJOBREQUEST._serialized_start=460
  _CREATEJOBREQUEST._serialized_end=502
  _UPDATEJOBREQUEST._serialized_start=504
  _UPDATEJOBREQUEST._serialized_end=595
  _JOBLOGSREQUEST._serialized_start=597
  _JOBLOGSREQUEST._serialized_end=692
  _JOBLOG._serialized_start=694
  _JOBLOG._serialized_end=779
  _UPDATERUNREQUEST._serialized_start=781
  _UPDATERUNREQUEST._serialized_end=903
  _WSJOBSERVICE._serialized_start=906
  _WSJOBSERVICE._serialized_end=1502
# @@protoc_insertion_point(module_scope)
